@@ -116,7 +116,7 @@ class FileStateMachine implements IStateMachine {
         $filename = $this->getFullFilePath($devid, $type, $key, $counter);
 
         if(file_exists($filename)) {
-            $contents = file_get_contents($filename);
+            $contents = Utils::SafeGetContents($filename, "GetState", false);
             $bytes = strlen($contents);
             ZLog::Write(LOGLEVEL_DEBUG, sprintf("FileStateMachine->GetState() read '%d' bytes from file: '%s'", $bytes, $filename ));
             return unserialize($contents);
@@ -205,7 +205,7 @@ class FileStateMachine implements IStateMachine {
 
         // exclusive block
         if ($mutex->Block()) {
-            $filecontents = @file_get_contents($this->userfilename);
+            $filecontents = Utils::SafeGetContents($this->userfilename, "LinkUserDevice", true);
 
             if ($filecontents)
                 $users = unserialize($filecontents);
@@ -249,7 +249,7 @@ class FileStateMachine implements IStateMachine {
 
         // exclusive block
         if ($mutex->Block()) {
-            $filecontents = @file_get_contents($this->userfilename);
+            $filecontents = Utils::SafeGetContents($this->userfilename, "UnLinkUserDevice", true);
 
             if ($filecontents)
                 $users = unserialize($filecontents);
@@ -301,7 +301,7 @@ class FileStateMachine implements IStateMachine {
             return $out;
         }
         else {
-            $filecontents = file_get_contents($this->userfilename);
+            $filecontents = Utils::SafeGetContents($this->userfilename, "GetAllDevices", false);
             if ($filecontents)
                 $users = unserialize($filecontents);
             else
@@ -323,7 +323,8 @@ class FileStateMachine implements IStateMachine {
      */
     public function GetStateVersion() {
         if (file_exists($this->settingsfilename)) {
-            $settings = unserialize(file_get_contents($this->settingsfilename));
+            $filecontents = Utils::SafeGetContents($this->settingsfilename, "GetStateVersion", false);
+            $settings = unserialize($filecontents);
             if (strtolower(gettype($settings) == "string") && strtolower($settings) == '2:1:{s:7:"version";s:1:"2";}') {
                 ZLog::Write(LOGLEVEL_INFO, "Broken state version file found. Attempt to autofix it. See https://jira.zarafa.com/browse/ZP-493 for more information.");
                 unlink($this->settingsfilename);
@@ -332,7 +333,7 @@ class FileStateMachine implements IStateMachine {
             }
         }
         else {
-            $filecontents = @file_get_contents($this->userfilename);
+            $filecontents = Utils::SafeGetContents($this->userfilename, "GetStateVersion", true);
             if ($filecontents)
                 $settings = array(self::VERSION => IStateMachine::STATEVERSION_01);
             else {
@@ -353,8 +354,10 @@ class FileStateMachine implements IStateMachine {
      * @return boolean
      */
     public function SetStateVersion($version) {
-        if (file_exists($this->settingsfilename))
-            $settings = unserialize(file_get_contents($this->settingsfilename));
+        if (file_exists($this->settingsfilename)){
+            $filecontents = Utils::SafeGetContents($this->settingsfilename, "SetStateVersion", false);
+            $settings = unserialize($filecontents);
+        }
         else
             $settings = array(self::VERSION => IStateMachine::STATEVERSION_01);
 
