@@ -28,6 +28,13 @@ class ZPushAdmin {
      * //TODO resync of a foldertype for all users (e.g. Appointment)
      */
 
+    const STATUS_SUCCESS = 0;
+    const STATUS_DEVICE_SYNCED_AFTER_DAYSOLD = 1;
+
+    public static $status = self::STATUS_SUCCESS;
+    public static $devices;
+    public static $userDevices;
+
     /**
      * List devices known to Z-Push.
      * If no user is given, all devices are listed
@@ -219,11 +226,13 @@ class ZPushAdmin {
      *
      * @param string    $user           (opt) user of the device
      * @param string    $devid          (opt) device id which should be removed
+     * @param int       $daysOld        (opt) devices which haven't synced for $daysOld days
+     * @param int       $time           (opt) unix timestamp to use with $daysOld
      *
      * @return boolean
      * @access public
      */
-    static public function RemoveDevice($user = false, $devid = false) {
+    static public function RemoveDevice($user = false, $devid = false, $daysOld = false, $time = false) {
         if ($user === false && $devid === false)
             return false;
 
@@ -232,7 +241,7 @@ class ZPushAdmin {
             $devicesIds = ZPush::GetStateMachine()->GetAllDevices($user);
             ZLog::Write(LOGLEVEL_DEBUG, sprintf("ZPushAdmin::RemoveDevice(): all '%d' devices for user '%s' found to be removed", count($devicesIds), $user));
             foreach ($devicesIds as $deviceid) {
-                if (!self::RemoveDevice($user, $deviceid)) {
+                if (!self::RemoveDevice($user, $deviceid, $daysOld, $time)) {
                     ZLog::Write(LOGLEVEL_ERROR, sprintf("ZPushAdmin::RemoveDevice(): removing devices failed for device '%s' of user '%s'. Aborting", $deviceid, $user));
                     return false;
                 }
@@ -243,7 +252,7 @@ class ZPushAdmin {
             $users = self::ListUsers($devid);
             ZLog::Write(LOGLEVEL_DEBUG, sprintf("ZPushAdmin::RemoveDevice(): device '%d' is used by '%d' users and will be removed", $devid, count($users)));
             foreach ($users as $aUser) {
-                if (!self::RemoveDevice($aUser, $devid)) {
+                if (!self::RemoveDevice($aUser, $devid, $daysOld, $time)) {
                     ZLog::Write(LOGLEVEL_ERROR, sprintf("ZPushAdmin::RemoveDevice(): removing user '%s' from device '%s' failed. Aborting", $aUser, $devid));
                     return false;
                 }
@@ -260,6 +269,21 @@ class ZPushAdmin {
                 $device->SetData($devicedata, false);
                 if (!isset($devicedata->devices))
                     throw new StateInvalidException("No devicedata stored in ASDevice");
+
+                if ($daysOld) {
+                    if (!$time) {
+                        $time = time();
+                    }
+                    $lastSynced = floor(($time - $device->getLastupdatetime()) / 86400);
+                    if ($daysOld > $lastSynced) {
+                        ZLog::Write(LOGLEVEL_INFO,
+                                sprintf("ZPushAdmin::RemoveDevice(): device '%s' of user '%s' synced %d day(s) ago but only devices which synced more than %d days ago will be removed. Skipping.",
+                                        $devid, $user, $lastSynced, $daysOld));
+                        self::$status = self::STATUS_DEVICE_SYNCED_AFTER_DAYSOLD;
+                        return true;
+                    }
+                }
+
                 $devices = $devicedata->devices;
             }
             catch (StateNotFoundException $e) {
@@ -296,16 +320,76 @@ class ZPushAdmin {
         return true;
     }
 
+    /**
+     * Sets options for a device.
+     * First argument is the FilterType.
+     * This value is superseeded by the globally configured SYNC_FILTERTIME_MAX.
+     * If set to false the value will not be modified in the device.
+     *
+     * @param string    $user           user of the device
+     * @param string    $devid          device id that should be modified
+     * @param int       $filtertype     SYNC_FILTERTYPE_1DAY to SYNC_FILTERTYPE_ALL, false to ignore
+     *
+     * @access public
+     * @return boolean
+     */
+    public static function SetDeviceOptions($user, $devid, $filtertype) {
+        if ($user === false || $devid === false) {
+            ZLog::Write(LOGLEVEL_ERROR, "ZPushAdmin::SetDeviceOptions(): user and device must be specified");
+            return false;
+        }
+
+        if ($filtertype !== false && $filtertype < SYNC_FILTERTYPE_ALL || $filtertype > SYNC_FILTERTYPE_INCOMPLETETASKS) {
+            ZLog::Write(LOGLEVEL_ERROR, sprintf("ZPushAdmin::SetDeviceOptions(): specified FilterType '%s' is out of bounds", $filtertype));
+            return false;
+        }
+
+        // load device data
+        $device = new ASDevice($devid, ASDevice::UNDEFINED, $user, ASDevice::UNDEFINED);
+        try {
+            $device->SetData(ZPush::GetStateMachine()->GetState($devid, IStateMachine::DEVICEDATA), false);
+        }
+        catch (StateNotFoundException $e) {
+            ZLog::Write(LOGLEVEL_ERROR, sprintf("ZPushAdmin::SetDeviceOptions(): device '%s' of user '%s' can not be found", $devid, $user));
+            return false;
+        }
+
+        if ($filtertype !== false) {
+            if ($filtertype === $device->GetSyncFilterType()) {
+                ZLog::Write(LOGLEVEL_DEBUG, sprintf("ZPushAdmin::SetDeviceOptions(): device '%s' of user '%s' - device FilterType already at '%s'. Terminating.", $devid, $user, $filtertype));
+            }
+            else {
+                $device->SetSyncFilterType($filtertype);
+            }
+        }
+
+        // save device data
+        if ($device->IsDataChanged()) {
+            try {
+                if ($device->IsNewDevice() || $device->GetData() === false) {
+                    ZLog::Write(LOGLEVEL_ERROR, sprintf("ZPushAdmin::SetDeviceOptions(): data of user '%s' not synchronized on device '%s'. Aborting.", $user, $devid));
+                    return false;
+                }
+                ZPush::GetStateMachine()->SetState($device->GetData(), $devid, IStateMachine::DEVICEDATA);
+                ZLog::Write(LOGLEVEL_DEBUG, sprintf("ZPushAdmin::SetDeviceOptions(): device '%s' of user '%s' - device FilterType updated to '%s'", $devid, $user, $filtertype));
+            }
+            catch (StateNotFoundException $e) {
+                ZLog::Write(LOGLEVEL_ERROR, sprintf("ZPushAdmin::SetDeviceOptions(): state for device '%s' of user '%s' can not be saved", $devid, $user));
+                return false;
+            }
+        }
+        return true;
+    }
 
     /**
-     * Marks a folder of a device of a user for re-synchronization
+     * Marks a folder of a device of a user for re-synchronization.
      *
      * @param string    $user           user of the device
      * @param string    $devid          device id which should be re-synchronized
      * @param mixed     $folderid       a single folder id or an array of folder ids
      *
-     * @return boolean
      * @access public
+     * @return boolean
      */
     static public function ResyncFolder($user, $devid, $folderid) {
         // load device data
@@ -480,6 +564,9 @@ class ZPushAdmin {
                 return false;
             }
 
+            // init deviceManage with correct device
+            ZPush::GetDeviceManagerWithDevice($device);
+
             // unify the lists saved for the user/device and the staticly configured one
             $new_list = array();
             foreach ($device->GetAdditionalFolders() as $folder) {
@@ -495,11 +582,12 @@ class ZPushAdmin {
                     $new_list[$fid] = array(
                                         'store' => $so->Store,
                                         'folderid' => $fid,
+                                        'parentid' => $so->parentid,
                                         'syncfolderid' => $syncfolderid,
                                         'name' => $so->displayname,
                                         'type' => $so->type,
                                         'origin' => ($syncfolderid !== $fid)?Utils::GetFolderOriginStringFromId($syncfolderid):'unknown',
-                                        'flags' => 0,           // static folders have no flags
+                                        'flags' => $so->Flags,
                                     );
                 }
             }
@@ -628,12 +716,6 @@ class ZPushAdmin {
                 return false;
             }
 
-            $static_folders = ZPush::GetAdditionalSyncFolders();
-            if (isset($static_folders[$add_folderid])) {
-                ZLog::Write(LOGLEVEL_ERROR, sprintf("ZPushAdmin::AdditionalFolderRemove(): the folder id '%s' can not be removed as it is a statically configured folder. Aborting.", $add_folderid));
-                return false;
-            }
-
             $status = $device->RemoveAdditionalFolder($add_folderid);
             if ($status && $device->GetData() !== false) {
                 ZPush::GetStateMachine()->SetState($device->GetData(), $devid, IStateMachine::DEVICEDATA);
@@ -737,18 +819,31 @@ class ZPushAdmin {
     }
 
     /**
-     * Fixes states with usernames in different cases
+     * Fixes states with usernames in different cases.
+     *
+     * @param string    $username
      *
      * @return boolean
      * @access public
      */
-    static public function FixStatesDifferentUsernameCases() {
+    static public function FixStatesDifferentUsernameCases($username=false) {
         $processed = 0;
         $dropedUsers = 0;
         $fixedUsers = 0;
+        $processedDevices = 0;
 
-        $devices = ZPush::GetStateMachine()->GetAllDevices(false);
+        if ($username) {
+            $devices = self::GetUserDevices($username);
+        }
+        else {
+            $devices = self::GetAllDevices();
+        }
+        $devicesCount = count($devices);
+        ZLog::Write(LOGLEVEL_DEBUG, sprintf("ZPushAdmin::FixStatesDifferentUsernameCases(): found %d devices", $devicesCount));
         foreach ($devices as $devid) {
+            if (defined('LOGFIXSTATES') && LOGFIXSTATES === true) {
+                ZLog::Write(LOGLEVEL_INFO, sprintf("ZPushAdmin::FixStatesDifferentUsernameCases(): Processing %d of %d. Device %s", ++$processedDevices, $devicesCount, $devid));
+            }
             $users = self::ListUsers($devid);
             $obsoleteUsers = array();
 
@@ -808,18 +903,30 @@ class ZPushAdmin {
     }
 
     /**
-     * Fixes states of available device data to the user linking
+     * Fixes states of available device data to the user linking.
+     *
+     * @param string    $username
      *
      * @return int
      * @access public
      */
-    static public function FixStatesDeviceToUserLinking() {
+    static public function FixStatesDeviceToUserLinking($username=false) {
         $seen = 0;
         $fixed = 0;
-        $devices = ZPush::GetStateMachine()->GetAllDevices(false);
-        ZLog::Write(LOGLEVEL_DEBUG, sprintf("ZPushAdmin::FixStatesDeviceToUserLinking(): found %d devices", count($devices)));
+        $processedDevices = 0;
 
+        if ($username) {
+            $devices = self::GetUserDevices($username);
+        }
+        else {
+            $devices = self::GetAllDevices();
+        }
+        $devicesCount = count($devices);
+        ZLog::Write(LOGLEVEL_DEBUG, sprintf("ZPushAdmin::FixStatesDeviceToUserLinking(): found %d devices", $devicesCount));
         foreach ($devices as $devid) {
+            if (defined('LOGFIXSTATES') && LOGFIXSTATES === true) {
+                ZLog::Write(LOGLEVEL_INFO, sprintf("ZPushAdmin::FixStatesDeviceToUserLinking(): Processing %d of %d. Device %s", ++$processedDevices, $devicesCount, $devid));
+            }
             $users = self::ListUsers($devid);
             foreach ($users as $username) {
                 $seen++;
@@ -834,19 +941,32 @@ class ZPushAdmin {
 
     /**
      * Fixes states of the user linking to the states
-     * and removes all obsolete states
+     * and removes all obsolete states.
+     *
+     * @param string    $username
      *
      * @return boolean
      * @access public
      */
-    static public function FixStatesUserToStatesLinking() {
+    static public function FixStatesUserToStatesLinking($username=false) {
         $processed = 0;
         $deleted = 0;
-        $devices = ZPush::GetStateMachine()->GetAllDevices(false);
-        ZLog::Write(LOGLEVEL_DEBUG, sprintf("ZPushAdmin::FixStatesUserToStatesLinking(): found %d devices", count($devices)));
-
+        $processedDevices = 0;
+        $processedStates = 0;
+        $deletedStates = 0;
+        if ($username) {
+            $devices = self::GetUserDevices($username);
+        }
+        else {
+            $devices = self::GetAllDevices();
+        }
+        $devicesCount = count($devices);
+        ZLog::Write(LOGLEVEL_DEBUG, sprintf("ZPushAdmin::FixStatesUserToStatesLinking(): found %d devices", $devicesCount));
         foreach ($devices as $devid) {
             try {
+                if (defined('LOGFIXSTATES') && LOGFIXSTATES === true) {
+                    ZLog::Write(LOGLEVEL_INFO, sprintf("ZPushAdmin::FixStatesUserToStatesLinking(): Processing %d of %d. Device %s", ++$processedDevices, $devicesCount, $devid));
+                }
                 // we work on device level
                 $devicedata = ZPush::GetStateMachine()->GetState($devid, IStateMachine::DEVICEDATA);
                 $knownUuids = array();
@@ -875,9 +995,13 @@ class ZPushAdmin {
             // get all uuids for deviceid from statemachine
             $existingStates = ZPush::GetStateMachine()->GetAllStatesForDevice($devid);
             $processed = count($existingStates);
-
-            ZLog::Write(LOGLEVEL_DEBUG, sprintf("ZPushAdmin::FixStatesUserToStatesLinking(): found %d valid uuids and %d states for device device '%s'", count($knownUuids), $processed, $devid));
-
+            $deleted = 0;
+            if (defined('LOGFIXSTATES') && LOGFIXSTATES === true) {
+                ZLog::Write(LOGLEVEL_INFO, sprintf("ZPushAdmin::FixStatesUserToStatesLinking(): found %d valid uuids and %d states for device '%s'", count($knownUuids), $processed, $devid));
+            }
+            else {
+                ZLog::Write(LOGLEVEL_DEBUG, sprintf("ZPushAdmin::FixStatesUserToStatesLinking(): found %d valid uuids and %d states for device '%s'", count($knownUuids), $processed, $devid));
+            }
             // remove states for all unknown uuids
             foreach ($existingStates as $obsoleteState) {
                 if ($obsoleteState['type'] === IStateMachine::DEVICEDATA)
@@ -886,31 +1010,47 @@ class ZPushAdmin {
                 if (!in_array($obsoleteState['uuid'], $knownUuids)) {
                     if (is_numeric($obsoleteState['counter']))
                         $obsoleteState['counter']++;
-
+                    if (defined('LOGFIXSTATES') && LOGFIXSTATES === true) {
+                        ZLog::Write(LOGLEVEL_INFO, sprintf("ZPushAdmin::FixStatesUserToStatesLinking(): obsoleteState TYPE '%s' UUID '%s' COUNTER %d for device '%s'", $obsoleteState['type'], $obsoleteState['uuid'], $obsoleteState['counter'], $devid));
+                    }
                     ZPush::GetStateMachine()->CleanStates($devid, $obsoleteState['type'], $obsoleteState['uuid'], $obsoleteState['counter']);
                     $deleted++;
                 }
             }
+            $processedStates +=  $processed;
+            $deletedStates += $deleted;
         }
-        return array($processed, $deleted);
+        return array($processedStates, $deletedStates);
     }
 
     /**
      * Fixes hierarchy states writing folderdata states.
      *
+     * @param string    $username
+     *
      * @access public
      * @return array(seenDevices, seenHierarchyStates, fixedHierarchyStates, usersWithoutHierarchy)
      */
-    static public function FixStatesHierarchyFolderData() {
+    static public function FixStatesHierarchyFolderData($username=false) {
         $devices = 0;
         $seen = 0;
         $nouuid = 0;
         $fixed = 0;
-        $asdevices = ZPush::GetStateMachine()->GetAllDevices(false);
-        ZLog::Write(LOGLEVEL_DEBUG, sprintf("ZPushAdmin::FixStatesHierarchyFolderData(): found %d devices", count($asdevices)));
+        $processedDevices = 0;
 
+        if ($username) {
+            $asdevices = self::GetUserDevices($username);
+        }
+        else {
+            $asdevices = self::GetAllDevices();
+        }
+        $devicesCount = count($asdevices);
+        ZLog::Write(LOGLEVEL_DEBUG, sprintf("ZPushAdmin::FixStatesHierarchyFolderData(): found %d devices", $devicesCount));
         foreach ($asdevices as $devid) {
             try {
+                if (defined('LOGFIXSTATES') && LOGFIXSTATES === true) {
+                    ZLog::Write(LOGLEVEL_INFO, sprintf("ZPushAdmin::FixStatesHierarchyFolderData(): Processing %d of %d. Device %s", ++$processedDevices, $devicesCount, $devid));
+                }
                 // get the device
                 $devicedata = ZPush::GetStateMachine()->GetState($devid, IStateMachine::DEVICEDATA);
                 $devices++;
@@ -975,18 +1115,30 @@ class ZPushAdmin {
     /**
      * Fixes missing flags or parentids on additional folders.
      *
+     * @param string    $username
+     *
      * @access public
      * @return array(seenDevices, devicesWithAdditionalFolders, fixedAdditionalFolders)
      */
-    static public function FixStatesAdditionalFolders() {
+    static public function FixStatesAdditionalFolders($username=false) {
         $devices = 0;
         $devicesWithAddFolders = 0;
         $fixed = 0;
-        $asdevices = ZPush::GetStateMachine()->GetAllDevices(false);
-        ZLog::Write(LOGLEVEL_DEBUG, sprintf("ZPushAdmin::FixStatesAdditionalFolders(): found %d devices", count($asdevices)));
+        $processedDevices = 0;
 
+        if ($username) {
+            $asdevices = self::GetUserDevices($username);
+        }
+        else {
+            $asdevices = self::GetAllDevices();
+        }
+        $devicesCount = count($asdevices);
+        ZLog::Write(LOGLEVEL_DEBUG, sprintf("ZPushAdmin::FixStatesAdditionalFolders(): found %d devices", $devicesCount));
         foreach ($asdevices as $devid) {
             try {
+                if (defined('LOGFIXSTATES') && LOGFIXSTATES === true) {
+                    ZLog::Write(LOGLEVEL_INFO, sprintf("ZPushAdmin::FixStatesAdditionalFolders(): Processing %d of %d. Device %s", ++$processedDevices, $devicesCount, $devid));
+                }
                 // get the device
                 $devicedata = ZPush::GetStateMachine()->GetState($devid, IStateMachine::DEVICEDATA);
                 $devices++;
@@ -1029,5 +1181,59 @@ class ZPushAdmin {
         return array($devices, $devicesWithAddFolders, $fixed);
     }
 
+    /**
+     * Returns the list of all devices.
+     *
+     * @access public
+     * @return array
+     */
+    public static function GetAllDevices() {
+        if (empty(self::$devices)) {
+            self::$devices = ZPush::GetStateMachine()->GetAllDevices(false);
+            if (defined('LOGFIXSTATES') && LOGFIXSTATES === true) {
+                ZLog::Write(LOGLEVEL_INFO, sprintf("ZPushAdmin::GetAllDevices(): found %d devices", count(self::$devices)));
+            }
+        }
+        return self::$devices;
+    }
+
+     /**
+     * Returns the list of all devices of a specific user.
+     *
+     * @access public
+     * @return array
+     */
+    public static function GetUserDevices($username) {
+        if (empty(self::$userDevices) && empty(self::$devices)) {
+            if (defined('LOGFIXSTATES') && LOGFIXSTATES === true){
+                ZLog::Write(LOGLEVEL_INFO, sprintf("ZPushAdmin::GetUserDevices(): Getting all devices for '%s'", $username));
+            }
+            $devices = self::GetAllDevices();
+            $devicesCount = count($devices);
+            self::$userDevices = array();
+            $processed = 0;
+
+            foreach ($devices as $devid) {
+                if (defined('LOGFIXSTATES') && LOGFIXSTATES === true){
+                    ZLog::Write(LOGLEVEL_INFO, sprintf("ZPushAdmin::GetUserDevices(): Processing %d on %d . Device %s", ++$processed , $devicesCount, $devid));
+                }
+                $users = self::ListUsers($devid);
+                foreach ($users as $user) {
+                    if ($user === $username){
+                        ZLog::Write(LOGLEVEL_INFO, sprintf("ZPushAdmin::GetUserDevices(): Found user '%s' into device %s", $username , $devid));
+                        self::$userDevices[] = $devid;
+                        continue;
+                    }
+                }
+            }
+
+            if (empty(self::$userDevices) && $username!==false && defined('LOGFIXSTATES') && LOGFIXSTATES === true){
+                ZLog::Write(LOGLEVEL_INFO, sprintf("ZPushAdmin::GetUserDevices(): No devices found for '%s'", $username));
+            }else if(defined('LOGFIXSTATES') && LOGFIXSTATES === true){
+                ZLog::Write(LOGLEVEL_INFO, sprintf("ZPushAdmin::GetUserDevices(): Found user '%s' into '%d' devices", $username , count(self::$userDevices)));
+            }
+        }
+        return self::$userDevices;
+    }
 
 }
