@@ -25,11 +25,11 @@
 
 include_once("syncworker.php");
 
-include_once('mapi/mapi.util.php');
-include_once('mapi/mapidefs.php');
-include_once('mapi/mapitags.php');
-include_once('mapi/mapicode.php');
-include_once('mapi/mapiguid.php');
+include_once('backend/kopano/mapi/mapi.util.php');
+include_once('backend/kopano/mapi/mapidefs.php');
+include_once('backend/kopano/mapi/mapitags.php');
+include_once('backend/kopano/mapi/mapicode.php');
+include_once('backend/kopano/mapi/mapiguid.php');
 
 if (!defined('PR_EMS_AB_THUMBNAIL_PHOTO')) {
     define('PR_EMS_AB_THUMBNAIL_PHOTO', mapi_prop_tag(PT_BINARY, 0x8C9E));
@@ -242,7 +242,7 @@ class Kopano extends SyncWorker {
         }
         else {
             $this->Log(sprintf("Kopano->clearAllNotCurrentChunkType: found %d invalid items, deleting", $querycnt));
-            $entries = mapi_table_queryallrows($table, array(PR_ENTRYID, $this->mapiprops['chunktype']));
+            $entries = mapi_table_queryrows($table, array(PR_ENTRYID, $this->mapiprops['chunktype']), 0, $querycnt);
             $entry_ids = array_reduce($entries, function ($result, $item) {
                                                     $result[] = $item[PR_ENTRYID];
                                                     return $result;
@@ -311,6 +311,24 @@ class Kopano extends SyncWorker {
 
         // get all the groups
         $groups = mapi_zarafa_getgrouplist($this->store, $ab_entryid);
+        $groupsUsers = array();
+        foreach ($groups as $groupDetails ) {
+            // All users are in group Everyone, so we don't have to get its members
+            if (strtoupper($groupDetails['groupname']) == 'EVERYONE') {
+                continue;
+            }
+            $groupentry = mapi_ab_openentry($addrbook, $groupDetails['groupid']);
+            $grouptable = @mapi_folder_getcontentstable($groupentry, MAPI_DEFERRED_ERRORS);
+            // some groups can not be listed - ZP-1196
+            if (mapi_last_hresult()) {
+                $this->Log(sprintf("Kopano->GetGAB(): Ignoring group '%s' as members can not be listed - possibly hidden, code: 0x%08X \n", $entry[PR_ACCOUNT], mapi_last_hresult() ));
+                continue;
+            }
+            $groupMembers = mapi_table_queryallrows($grouptable, array(PR_ACCOUNT));
+            if (!empty($groupMembers)) {
+                $groupsUsers[$groupDetails['groupname']] = $this->arrayFlatten($groupMembers, PR_ACCOUNT);
+            }
+        }
 
         // restrict the table if we should only return one
         if ($uniqueId) {
@@ -325,8 +343,9 @@ class Kopano extends SyncWorker {
                 $this->Terminate(sprintf("Kopano->getGAB(): Single GAB entry '%s' requested but %d entries found. Aborting.", $uniqueId, $querycnt));
             }
         }
+        $querycnt = mapi_table_getrowcount($table);
 
-        $gabentries = mapi_table_queryallrows($table, array(PR_ENTRYID,
+        $gabentries = mapi_table_queryrows($table, array(   PR_ENTRYID,
                                                             PR_ACCOUNT,
                                                             PR_DISPLAY_NAME,
                                                             PR_SMTP_ADDRESS,
@@ -350,7 +369,7 @@ class Kopano extends SyncWorker {
                                                             PR_EMS_AB_THUMBNAIL_PHOTO,
                                                             PR_EC_AB_HIDDEN,
                                                             PR_DISPLAY_TYPE_EX
-                                                    ));
+                                           ), 0, $querycnt);
         if(!is_array($gabentries)) {
             $this->Log("Kopano->GetGAB(): GAB data can not be retrieved.");
             return $data;
@@ -365,14 +384,20 @@ class Kopano extends SyncWorker {
                 $this->Log(sprintf("Kopano->GetGAB(): Ignoring user '%s' as account is hidden", $entry[PR_ACCOUNT]));
                 continue;
             }
+            // ignore if user sync is disabled
+            if (isset($entry[PR_DISPLAY_TYPE_EX]) && $entry[PR_DISPLAY_TYPE_EX] == DT_MAILUSER) {
+                if (! (GAB_SYNC_TYPES & GAB_SYNC_USER)) {
+                    continue;
+                }
+            }
 
             $a = new GABEntry();
             $a->type = GABEntry::CONTACT;
-            $a->memberOf = array();
-            $memberOf = mapi_zarafa_getgrouplistofuser($this->store, $entry[PR_ENTRYID]);
-            if (is_array($memberOf)) {
-                $a->memberOf = array_keys($memberOf);
+            if (strtoupper($entry[PR_DISPLAY_NAME]) != "EVERYONE") {
+                // Avoid loop because all other entries are members of Everyone group
+                $a->memberOf = $this->getMemberOf($groupsUsers, $entry[PR_ACCOUNT]);
             }
+
             // the company name is 'Everyone'
             if ($gabId != null && $entry[PR_DISPLAY_NAME] == $gabName) {
                 $entry[PR_ACCOUNT] = "Everyone";
@@ -380,6 +405,9 @@ class Kopano extends SyncWorker {
             }
             // is this a group?
             if (array_key_exists($entry[PR_ACCOUNT], $groups)) {
+                if (! (GAB_SYNC_TYPES & GAB_SYNC_GROUP)) {
+                    continue;
+                }
                 $a->type = GABEntry::GROUP;
                 $groupentry = mapi_ab_openentry($addrbook, $entry[PR_ENTRYID]);
                 $grouptable = @mapi_folder_getcontentstable($groupentry, MAPI_DEFERRED_ERRORS);
@@ -404,10 +432,22 @@ class Kopano extends SyncWorker {
                     }
                 }
             }
-            else if (isset($entry[PR_DISPLAY_TYPE_EX]) && $entry[PR_DISPLAY_TYPE_EX] == DT_ROOM) {
+            elseif (isset($entry[PR_DISPLAY_TYPE_EX]) && $entry[PR_DISPLAY_TYPE_EX] == DT_REMOTE_MAILUSER) {
+                if (! (GAB_SYNC_TYPES & GAB_SYNC_CONTACT)) {
+                    continue;
+                }
+                $a->type = GABEntry::CONTACT;
+            }
+            elseif (isset($entry[PR_DISPLAY_TYPE_EX]) && $entry[PR_DISPLAY_TYPE_EX] == DT_ROOM) {
+                if (! (GAB_SYNC_TYPES & GAB_SYNC_ROOM)) {
+                    continue;
+                }
                 $a->type = GABEntry::ROOM;
             }
-            else if (isset($entry[PR_DISPLAY_TYPE_EX]) && $entry[PR_DISPLAY_TYPE_EX] ==  DT_EQUIPMENT) {
+            elseif (isset($entry[PR_DISPLAY_TYPE_EX]) && $entry[PR_DISPLAY_TYPE_EX] == DT_EQUIPMENT) {
+                if (! (GAB_SYNC_TYPES & GAB_SYNC_EQUIPMENT)) {
+                    continue;
+                }
                 $a->type = GABEntry::EQUIPMENT;
             }
 
@@ -497,6 +537,19 @@ class Kopano extends SyncWorker {
             return false;
         }
         $chunkdata = $this->findChunk($store, $folderid, $chunkName);
+        // remove chunk if it's empty
+        if ($amountEntries == 0 && !empty($chunkdata)) {
+            $this->Log(sprintf("Kopano->setChunkData: %s - removing empty chunk", $chunkName));
+            $folder = $this->getFolder($store, $folderid);
+            mapi_folder_deletemessages($folder, array($chunkdata[PR_ENTRYID]), DELETE_HARD_DELETE);
+            if (mapi_last_hresult()) {
+                $this->Log("Kopano->setChunkData: error deleting empty chunk from the GAB folder: 0x%08X", mapi_last_hresult());
+            }
+            return true;
+        }
+        elseif ($amountEntries == 0) {
+            return true;
+        }
         $message = false;
 
         // message not found, create it
@@ -543,7 +596,7 @@ class Kopano extends SyncWorker {
         }
 
         // output log
-        $this->log($log);
+        $this->Log($log);
         return true;
     }
 
@@ -567,7 +620,7 @@ class Kopano extends SyncWorker {
         $folder = $this->getFolder($store, $folderid);
         $table = mapi_folder_getcontentstable($folder);
         if (!$table)
-            $this->Log(sprintf("Kopano->findChunk: Error, unable to read contents table to find chunk '%d': 0x%08X", $chunkId, mapi_last_hresult()));
+            $this->Log(sprintf("Kopano->findChunk: Error, unable to read contents table to find chunk '%s': 0x%08X", $chunkName, mapi_last_hresult()));
 
         $restriction = array(RES_PROPERTY, array(RELOP => RELOP_EQ, ULPROPTAG => PR_SUBJECT, VALUE => $chunkName));
         mapi_table_restrict($table, $restriction);
@@ -764,7 +817,7 @@ class Kopano extends SyncWorker {
             return "";
         }
         elseif ($ret) {
-            $this->Log("Kopano->readPropStream error opening stream: 0x%08X", $ret);
+            $this->Log(sprintf("Kopano->readPropStream error opening stream: 0x%08X", $ret));
             return "";
         }
         $data = "";
@@ -804,4 +857,54 @@ class Kopano extends SyncWorker {
 
         return true;
     }
+
+    /**
+     * Returns the array of groups in which the user is in.
+     *
+     * @param array     $groupsUsers    list of groups and their users
+     * @param string    $user           username
+     *
+     * @access private
+     * @return array
+     */
+    private function getMemberOf($groupsUsers, $user) {
+        // every user is in Everyone group
+        $groups = array('Everyone');
+        foreach ($groupsUsers as $group => $userDetails) {
+            if (in_array($user, $userDetails)) {
+                $groups[] = $group;
+                continue;
+            }
+        }
+        return $groups;
+    }
+
+    /**
+     * Flattens multi-dimensional array similar to array_column() function
+     * which is available since PHP 5.5.
+     *
+     * Input:
+     * Array (
+     *  [0] => Array([973078558] => entry0)
+     *  [1] => Array([973078558] => entry1)
+     *  [2] => Array([973078558] => entry2)
+     *  [3] => Array([973078558] => entry3)
+     *  [4] => Array([973078558] => entry4)
+     * )
+     * Result:
+     * Array (
+     *  [0] => entry0
+     *  [1] => entry1
+     *  [2] => entry2
+     *  [3] => entry3
+     *  [4] => entry4
+     * )
+     *
+     * @param unknown $array
+     * @param unknown $column_name
+     * @return array
+     */
+    private function arrayFlatten($array, $key) {
+        return array_map(function($element) use($key) {return $element[$key];}, $array);
+    }
 }
\ No newline at end of file
